/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.model.sweden.impl;

import java.util.Date;

import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Formula;

/**
 * The Class ParliamentMemberBallotRecord.
 */
@Entity
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@DiscriminatorValue("ParliamentMemberBallotRecord")
public class ParliamentMemberBallotRecord extends AbstractBallotMetaData {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -7843524383033778447L;

    /** The first vote date. */
    private Date firstVoteDate = new Date();

    /** The last vote date. */
    private Date lastVoteDate = new Date(0L);

    /** The opponent. */
    private long opponent = 0;

    /** The percentage absent. */
    private long percentageAbsent;

    /** The percentage loosing. */
    private long percentageLoosing;

    /** The percentage loyal. */
    private long percentageLoyal;

    /** The percentage present. */
    private long percentagePresent;

    /** The percentage rebel. */
    private long percentageRebel;

    /** The percentage winning. */
    private long percentageWinning;

    /** The rebel. */
    private long rebel = 0;
    
    
    /**
     * Instantiates a new parliament member ballot record.
     */
    public ParliamentMemberBallotRecord() {
		super();
	}
    
    /**
     * Gets the first vote date.
     *
     * @return the first vote date
     */
    @Temporal(TemporalType.DATE)
    public Date getFirstVoteDate() {
        return firstVoteDate;
    }

    /**
     * Gets the last vote date.
     *
     * @return the last vote date
     */
    @Temporal(TemporalType.DATE)
    public Date getLastVoteDate() {
        return lastVoteDate;
    }

    /**
     * Gets the loyal.
     *
     * @return the loyal
     */
    @Transient
    public long getLoyal() {
        return (totalVotes - neutralVotes - rebel);
    }

    /**
     * Gets the opponent.
     *
     * @return the opponent
     */
    public long getOpponent() {
        return opponent;
    }

    /**
     * Gets the percentage absent.
     *
     * @return the percentage absent
     */
    @Formula("(absentVotes * 100) / totalVotes")
    public long getPercentageAbsent() {
        return percentageAbsent;
    }

    /**
     * Gets the percentage loosing.
     *
     * @return the percentage loosing
     */
    @Formula("(opponent * 100) / totalVotes")
    public long getPercentageLoosing() {
        return percentageLoosing;
    }

    /**
     * Gets the percentage loyal.
     *
     * @return the percentage loyal
     */
    @Formula("(totalVotes - neutralVotes - rebel)*100/totalVotes")
    public long getPercentageLoyal() {
        return percentageLoyal;
    }

    /**
     * Gets the percentage present.
     *
     * @return the percentage present
     */
    @Formula("(totalVotes - absentVotes) * 100/ totalVotes")
    public long getPercentagePresent() {
        return percentagePresent;
    }

    /**
     * Gets the percentage rebel.
     *
     * @return the percentage rebel
     */

    @Formula("(rebel * 100)/ totalVotes")
    public long getPercentageRebel() {
        return percentageRebel;
    }

    /**
     * Gets the percentage winning.
     *
     * @return the percentage winning
     */
    @Formula("(totalVotes -neutralVotes -absentVotes -opponent)* 100/ totalVotes")
    public long getPercentageWinning() {
        return percentageWinning;
    }

    /**
     * Gets the present.
     *
     * @return the present
     */
    @Transient
    public long getPresent() {
        return (totalVotes - absentVotes);
    }
    
    /**
     * Gets the rebel.
     *
     * @return the rebel
     */
    public long getRebel() {
        return rebel;
    }
    
    /**
     * Gets the winning.
     *
     * @return the winning
     */
    @Transient
    public long getWinning() {
        return (totalVotes - neutralVotes - absentVotes - opponent);
    }

    /* (non-Javadoc)
     * @see com.hack23.cia.model.sweden.impl.AbstractBallotMetaData#newVote(com.hack23.cia.model.sweden.impl.Vote)
     */
    @Override
	@Transient
    public void newVote(Vote vote) {
    	super.newVote(vote);

        if (vote.getDatum().before(getFirstVoteDate())) {
            setFirstVoteDate(vote.getDatum());
        }

        if (vote.getDatum().after(getLastVoteDate())) {
            setLastVoteDate(vote.getDatum());
        }
    }

    /**
     * Sets the first vote date.
     *
     * @param firstVoteDate the new first vote date
     */
    public void setFirstVoteDate(Date firstVoteDate) {
        this.firstVoteDate = firstVoteDate;
    }

    /**
     * Sets the last vote date.
     *
     * @param lastVoteDate the new last vote date
     */
    public void setLastVoteDate(Date lastVoteDate) {
        this.lastVoteDate = lastVoteDate;
    }

    /**
     * Sets the opponent.
     *
     * @param opponent the new opponent
     */
    public void setOpponent(long opponent) {
        this.opponent = opponent;
    }

    /**
     * Sets the percentage absent.
     *
     * @param percentageAbsent the new percentage absent
     */
    public void setPercentageAbsent(long percentageAbsent) {
        this.percentageAbsent = percentageAbsent;
    }

    /**
     * Sets the percentage loosing.
     *
     * @param percentageLoosing the new percentage loosing
     */
    public void setPercentageLoosing(long percentageLoosing) {
        this.percentageLoosing = percentageLoosing;
    }

    /**
     * Sets the percentage loyal.
     *
     * @param percentageLoyal the new percentage loyal
     */
    public void setPercentageLoyal(long percentageLoyal) {
        this.percentageLoyal = percentageLoyal;
    }

    /**
     * Sets the percentage present.
     *
     * @param percentagePresent the new percentage present
     */
    public void setPercentagePresent(long percentagePresent) {
        this.percentagePresent = percentagePresent;
    }

    /**
     * Sets the percentage rebel.
     *
     * @param percentageRebel the new percentage rebel
     */
    public void setPercentageRebel(long percentageRebel) {
        this.percentageRebel = percentageRebel;
    }

    /**
     * Sets the percentage winning.
     *
     * @param percentageWinning the new percentage winning
     */
    public void setPercentageWinning(long percentageWinning) {
        this.percentageWinning = percentageWinning;
    }


    /**
     * Sets the rebel.
     *
     * @param rebel the new rebel
     */
    public void setRebel(long rebel) {
        this.rebel = rebel;
    }

}
