/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.model.sweden.impl;

import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;
import javax.persistence.Version;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import com.hack23.cia.model.core.impl.BaseEntity;

/**
 * The Class Vote.
 */
@Entity
@Table(uniqueConstraints = { @UniqueConstraint(columnNames = {
        "parliamentMember_id", "ballot_id" }) })
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Vote extends BaseEntity {

    /**
     * The Enum Position.
     */
    public enum Position {

        /** The Absent. */
        Absent,

        /** The Neutral. */
        Neutral,

        /** The No. */
        No,

        /** The Yes. */
        Yes
    }

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The ballot. */
    private Ballot ballot;

    /** The datum. */
    private Date datum;

    /** The id. */
    private Long id;

    /** The parliament member. */
    private ParliamentMember parliamentMember;

    /** The position. */
    private Position position;
    
    /** The version. */
    private Long version=1L;

    /** The vote meta data. */
    private VoteMetaData voteMetaData;

    /**
     * Gets the ballot.
     *
     * @return the ballot
     */
    @ManyToOne
    public Ballot getBallot() {
        return ballot;
    }

    /**
     * Gets the datum.
     *
     * @return the datum
     */
    @Temporal(TemporalType.DATE)
    public Date getDatum() {
        return datum;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.hack23.cia.model.core.BaseEntity#getId()
     */
    @Override
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Long getId() {
        return id;
    }

    /**
     * Gets the parliament member.
     *
     * @return the parliament member
     */
    @ManyToOne
    public ParliamentMember getParliamentMember() {
        return parliamentMember;
    }

    /**
     * Gets the position.
     *
     * @return the position
     */
    @Enumerated(EnumType.STRING)
    public Position getPosition() {
        return position;
    }

    /* (non-Javadoc)
     * @see com.hack23.cia.model.core.BaseEntity#getVersion()
     */
    @Override
    @Version
    public Long getVersion() {
        return version;
    }

    /**
     * Gets the vote meta data.
     *
     * @return the vote meta data
     */
    @OneToOne(cascade=CascadeType.ALL)
    @JoinColumn(name="id")
	public VoteMetaData getVoteMetaData() {
		return voteMetaData;
	}

    /**
     * Checks if is rebel.
     *
     * @return true, if is rebel
     */
    @Transient
    public boolean isRebel() {
    	return getBallot().isRebelVote(this);
    }

    /**
     * Checks if is winning.
     *
     * @return true, if is winning
     */
    @Transient
    public boolean isWinning() {
    	return getBallot().isWinningPosition(this.getPosition());
    }

    /**
     * Sets the ballot.
     *
     * @param ballot the new ballot
     */
    public void setBallot(Ballot ballot) {
        this.ballot = ballot;
    }

    /**
     * Sets the datum.
     *
     * @param datum the new datum
     */
    public void setDatum(Date datum) {
        this.datum = datum;
    }

    /**
     * Sets the id.
     *
     * @param id the new id
     */
    public void setId(Long id) {
        this.id = id;
    }
    
    /**
     * Sets the parliament member.
     *
     * @param parliamentMember the new parliament member
     */
    public void setParliamentMember(ParliamentMember parliamentMember) {
        this.parliamentMember = parliamentMember;
    }

    /**
     * Sets the position.
     *
     * @param position the new position
     */
    public void setPosition(Position position) {
        this.position = position;
    }

    /**
     * Sets the version.
     *
     * @param version the new version
     */
    public void setVersion(Long version) {
        this.version = version;
    }

	/**
	 * Sets the vote meta data.
	 *
	 * @param voteMetaData the new vote meta data
	 */
	public void setVoteMetaData(VoteMetaData voteMetaData) {
		this.voteMetaData = voteMetaData;
	}
	
}
