/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.service.impl.agent.sweden;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;
import com.hack23.cia.model.sweden.impl.Ballot;
import com.hack23.cia.model.sweden.impl.CommitteeReport;
import com.hack23.cia.model.sweden.impl.ParliamentMember;
import com.hack23.cia.model.sweden.impl.Vote;

/**
 * The Class BallotAgentImpl.
 */
public class BallotAgentImpl implements BallotAgent {


    /** The Constant ABSENT. */
    private static final String ABSENT = "Frånvarande"; //$NON-NLS-1$

    /** The Constant ABSENT_STYLE2. */
    private static final String ABSENT_STYLE2 = "Avstående"; //$NON-NLS-1$

    /** The Constant ANCHOR. */
    private static final String ANCHOR = "a"; //$NON-NLS-1$

    /** The Constant COMMITEE_REPORT_AND_BALLOTS. */
    private static final String COMMITEE_REPORT_AND_BALLOTS = "Utskottets förslag och kammarens omröstning"; //$NON-NLS-1$

    /** The Constant HTTP_WWW_RIKSDAGEN_SE. */
    private static final String HTTP_WWW_RIKSDAGEN_SE = "http://www.riksdagen.se"; //$NON-NLS-1$

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory
            .getLog(BallotAgentImpl.class);

    /** The Constant NEUTRAL. */
    private static final String NEUTRAL = "Avstår"; //$NON-NLS-1$

    /** The Constant NO. */
    private static final String NO = "Nej"; //$NON-NLS-1$

    /** The Constant SHOW_PARLIAMENT_MEMBERS_VOTES. */
    private static final String SHOW_PARLIAMENT_MEMBERS_VOTES = "Visa ledamöternas röster"; //$NON-NLS-1$

    /** The Constant TABLE. */
    private static final String TABLE = "table"; //$NON-NLS-1$

    /** The Constant YES. */
    private static final String YES = "Ja"; //$NON-NLS-1$

    /** The web client. */
    private final WebClient webClient;

    /**
     * Instantiates a new ballot agent impl.
     *
     * @param webClient the web client
     */
    public BallotAgentImpl(final WebClient webClient) {
        super();
        this.webClient = webClient;
        this.webClient.setJavaScriptEnabled(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.CommiteeReportAgent#findRöstadeballot
     * (com.hack23.cia.model.sweden.CommiteeReport)
     */
    public final List<Ballot> findBallots(final CommitteeReport commiteeReport) {
        try {
            HtmlPage page = (HtmlPage) webClient.getPage(commiteeReport
                    .getHref());

            List<HtmlAnchor> anchors = page.getDocumentElement()
                    .getHtmlElementsByTagName(ANCHOR);

            HtmlAnchor findVoteAnchor = findVoteAnchor(anchors);

            if (findVoteAnchor != null) {
                return findVotedballots(findVoteAnchor);
            }

        } catch (Exception e) {
            LOGGER.warn(e);
        }
        return null;
    }

    /**
     * Find vote anchor.
     *
     * @param anchors the anchors
     * @return the html anchor
     */
    private HtmlAnchor findVoteAnchor(final List<HtmlAnchor> anchors) {
        for (HtmlAnchor anchor : anchors) {
            if (COMMITEE_REPORT_AND_BALLOTS.equals(anchor.asText())) {
                return anchor;
            }
        }
        return null;
    }

    /**
     * Find votedballots.
     *
     * @param votePage the vote page
     * @return the list
     */
    private List<Ballot> findVotedballots(final HtmlAnchor votePage) {
        List<Ballot> resultat = new ArrayList<Ballot>();
        HtmlPage page;
        try {
            page = (HtmlPage) votePage.click();
            List<HtmlAnchor> anchors = page.getDocumentElement()
                    .getHtmlElementsByTagName(ANCHOR);
            List<HtmlTable> tables = page.getDocumentElement()
                    .getHtmlElementsByTagName(TABLE);
            int index = 0;

            for (HtmlAnchor anchor : anchors) {
                if (SHOW_PARLIAMENT_MEMBERS_VOTES.equals(anchor.asText())) {

                    Ballot ballot = new Ballot();
                    ballot.setDescription(tables.get(index).getRow(0).asText());
                    ballot.setOverviewHref(votePage.getHrefAttribute());
                    ballot.setVoteResultsHref(HTTP_WWW_RIKSDAGEN_SE
                            + anchor.getHrefAttribute());

                    resultat.add(ballot);
                    index++;
                }
            }

        } catch (IOException e) {
            LOGGER.warn("Problem accessing Votes", e); //$NON-NLS-1$
        }
        return resultat;
    }


    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.CommiteeReportAgent#getVoteResult
     * (com.hack23.cia.model.sweden.Ballot)
     */
    public final List<Vote> getVoteResult(final Ballot ballot) {
        List<Vote> resultat = new ArrayList<Vote>();
        try {
            HtmlPage page = (HtmlPage) webClient.getPage(ballot
                    .getVoteResultsHref());
            HtmlTable table = (HtmlTable) page.getDocumentElement()
                    .getHtmlElementsByTagName(TABLE).iterator().next();

            List<HtmlTableRow> rows = table.getRows();

            try {
                for (int i = 1; i < rows.size(); i++) {
                    HtmlTableRow row = rows.get(i);
                    String name = row.getCell(0).asText().trim();
                    String party = row.getCell(1).asText();
                    String electoralArea = row.getCell(2).asText();
                    String voteStr = row.getCell(3).asText().trim();

                    ParliamentMember parliamentMember = new ParliamentMember();
                    parliamentMember.setName(name);
                    parliamentMember.setParty(party);
                    parliamentMember.setElectoralRegion(electoralArea);
                    parliamentMember.createFuzzyKey();

                    Vote vote = new Vote();
                    vote.setParliamentMember(parliamentMember);

                    if (voteStr.equalsIgnoreCase(YES)) {
                        vote.setPosition(Vote.Position.Yes);
                    } else if (voteStr.equalsIgnoreCase(NO)) {
                        vote.setPosition(Vote.Position.No);
                    } else if (voteStr.equalsIgnoreCase(ABSENT)) {
                        vote.setPosition(Vote.Position.Absent);
                    } else if (voteStr.equalsIgnoreCase(NEUTRAL)) {
                        vote.setPosition(Vote.Position.Neutral);
                    } else if (voteStr.equalsIgnoreCase(ABSENT_STYLE2)) {
                        vote.setPosition(Vote.Position.Neutral);
                    }

                    resultat.add(vote);
                }
            } catch (IndexOutOfBoundsException ie) {
                LOGGER.info("Vote result missing: " //$NON-NLS-1$
                        + ballot.getVoteResultsHref()
                        + "  for " + ballot.getOverviewHref()); //$NON-NLS-1$
            }
        } catch (Exception e) {
            LOGGER.warn("warn", e); //$NON-NLS-1$
        }
        return resultat;
    }

}
